@@ -42,14 +42,21 @@ as_nanoarrow_buffer.nanoarrow_buffer <- function(x, ...) {
 
 #' @export
 as_nanoarrow_buffer.default <- function(x, ...) {
+  msg <- NULL
   result <- tryCatch(
     .Call(nanoarrow_c_as_buffer_default, x),
-    error = function(...) NULL
+    error = function(e) {
+      msg <<- conditionMessage(e)
+      NULL
+    }
   )
 
-  if (is.null(result)) {
+  if (is.null(result) && is.null(msg)) {
     cls <- paste(class(x), collapse = "/")
     stop(sprintf("Can't convert object of type %s to nanoarrow_buffer", cls))
+  } else if (is.null(result)) {
+    cls <- paste(class(x), collapse = "/")
+    stop(sprintf("Can't convert object of type %s to nanoarrow_buffer: %s", cls, msg))
   }
 
   result
@@ -176,7 +183,7 @@ convert_buffer <- function(buffer, to = NULL) {
 #' @export
 as_nanoarrow_array.nanoarrow_buffer <- function(x, ..., schema = NULL) {
   if (!is.null(schema)) {
-    stop("as_nanoarrow_array(<nanoarow_buffer>) with non-NULL schema is not supported")
+    stop("as_nanoarrow_array(<nanoarrow_buffer>) with non-NULL schema is not supported")
   }
 
   info <- nanoarrow_buffer_info(x)
